@@ -95,7 +95,7 @@ class ModuleEntry(ClientEntry):
             self._module = None
             logger.warning("Could not compile @%r", self.subpath)
             await self.root.err.record_error(
-                "compile", self.subpath, exc=exc, reason="compilation", message="compiler error"
+                "compile", self.subpath, exc=exc, message="compiler error"
             )
         else:
             await self.root.err.record_working("compile", self.subpath)
@@ -183,6 +183,10 @@ class CodeEntry(ClientEntry):
     _code = None
     is_async = None
 
+    def __init__(self, *a, **kv):
+        self.reload_event = anyio.create_event()
+        super().__init__(*a, *kv)
+
     @property
     def name(self):
         return P(self.subpath)
@@ -201,13 +205,15 @@ class CodeEntry(ClientEntry):
         except Exception as exc:
             logger.warning("Could not compile @%s", self.subpath)
             await self.root.err.record_error(
-                "compile", self.subpath, exc=exc, reason="compilation", message="compiler error"
+                "compile", self.subpath, exc=exc, message="compiler error"
             )
             self._code = None
         else:
             await self.root.err.record_working("compile", self.subpath)
             self._code = p
             self.is_async = a
+            await self.reload_event.set()
+            self.reload_event = anyio.create_event()
 
     def __call__(self, *a, **kw):
         if self._code is None:
