@@ -9,7 +9,7 @@ from functools import partial
 
 from distkv.code import CodeRoot
 from distkv.runner import AnyRunnerRoot, SingleRunnerRoot, AllRunnerRoot
-from distkv.util import yprint, PathLongener, P, Path, data_get
+from distkv.util import yprint, PathLongener, P, Path, data_get, attrdict
 
 import logging
 
@@ -103,9 +103,19 @@ def _state_fix_2(rs):
             )
     except AttributeError:
         pass
+    try:
+        if rs.computed:
+            try:
+                rs.computed_date = datetime.datetime.fromtimestamp(rs.computed).strftime(
+                    "%Y-%m-%d %H:%M:%S"
+                )
+            except (ValueError, OverflowError):
+                pass
+    except AttributeError:
+        pass
 
 
-async def _state_fix(obj, state, r):
+async def _state_fix(obj, state, path, r):
     try:
         val = r.value
     except AttributeError:
@@ -120,6 +130,8 @@ async def _state_fix(obj, state, r):
             val["state"] = rs.value
         if "value" in rs:
             _state_fix_2(rs.value)
+    if path:
+        r.path = path + r.path
     try:
         val.target_date = datetime.datetime.fromtimestamp(val.target).strftime("%Y-%m-%d %H:%M:%S")
     except (AttributeError, TypeError):
@@ -130,6 +142,7 @@ async def _state_fix(obj, state, r):
 
 @cli.command("list")
 @click.option("-s", "--state", is_flag=True, help="Add state data")
+@click.option("-t", "--table", is_flag=True, help="one-line output")
 @click.option(
     "-d",
     "--as-dict",
@@ -139,9 +152,12 @@ async def _state_fix(obj, state, r):
 )
 @click.argument("path", nargs=1)
 @click.pass_obj
-async def list_(obj, state, as_dict, path):
+async def list_(obj, state, table, as_dict, path):
     """List run entries.
     """
+    if table and state:
+        click.UsageError("'--table' and '--state' are mutually exclusive")
+
     path = P(path)
     if obj.subpath[-1] == "-":
         if path:
@@ -157,11 +173,51 @@ async def list_(obj, state, as_dict, path):
             print(r.path[-1], file=obj.stdout)
         return
 
-    if state:
+    if state or table:
         state = obj.statepath + path
-    path = obj.path + path
 
-    await data_get(obj, path, as_dict=as_dict, item_mangle=partial(_state_fix, obj, state))
+    if table:
+        from distkv.errors import ErrorRoot
+
+        err = await ErrorRoot.as_handler(obj.client)
+
+        async for r in obj.client.get_tree(obj.path + path):
+            p = path + r.path
+            s = await obj.client.get(state + r.path)
+            if "value" not in s:
+                st = "-never-"
+            elif s.value.started > s.value.stopped:
+                st = s.value.node
+            else:
+                try:
+                    e = await err.get_error_record("run", obj.path + p, create=False)
+                except KeyError:
+                    st = "-stopped-"
+                else:
+                    if e is None or e.resolved:
+                        st = "-stopped-"
+                    else:
+                        st = " | ".join(
+                            "%s %s"
+                            % (
+                                Path.build(e.subpath)
+                                if e._path[-2] == ee._path[-1]
+                                else Path.build(ee.subpath),
+                                getattr(ee, "message", None)
+                                or getattr(ee, "comment", None)
+                                or "-",
+                            )
+                            for ee in e
+                        )
+            print(p, r.value.code, st, file=obj.stdout)
+
+    else:
+        await data_get(
+            obj,
+            obj.path + path,
+            as_dict=as_dict,
+            item_mangle=partial(_state_fix, obj, state, None if as_dict else path),
+        )
 
 
 @cli.command("state")
@@ -212,7 +268,7 @@ async def get(obj, path, state):
     res.path = path
     if state:
         state = obj.statepath
-    await _state_fix(obj, state, res)
+    await _state_fix(obj, state, res, None)
     if not obj.meta:
         res = res.value
 
@@ -238,11 +294,11 @@ async def delete(obj, path, force):
         val = res.value
         if val.target is not None:
             val.target = None
-            res = await obj.client.set(obj.path + path, nchain=3, chain=res.chain)
+            res = await obj.client.set(obj.path + path, value=val, nchain=3, chain=res.chain)
             if not force:
                 res.info = "'target' was set: cleared but not deleted."
         if force or val.target is None:
-            sres = await obj.client.get(obj.statepath + path, nchain=obj.meta)
+            sres = await obj.client.get(obj.statepath + path, nchain=3)
             if not force and "value" in sres and sres.value.stopped < sres.value.started:
                 res.info = "Still running, not deleted."
             else:
@@ -255,7 +311,7 @@ async def delete(obj, path, force):
 
     if obj.meta:
         yprint(res, stream=obj.stdout)
-    else:
+    elif obj.debug:
         print(res.info)
 
 
@@ -264,25 +320,25 @@ async def delete(obj, path, force):
 @click.option("-C", "--copy", help="Use this entry as a template.")
 @click.option("-t", "--time", "tm", help="time the code should next run at. '-':not")
 @click.option("-r", "--repeat", type=int, help="Seconds the code should re-run after")
-@click.option("-k", "--ok", type=int, help="Code is OK if it ran this many seconds")
+@click.option("-k", "--ok", type=float, help="Code is OK if it ran this many seconds")
 @click.option("-b", "--backoff", type=float, help="Back-off factor. Default: 1.4")
 @click.option("-d", "--delay", type=int, help="Seconds the code should retry after (w/ backoff)")
 @click.option("-i", "--info", help="Short human-readable information")
-@click.option("-v", "--var", nargs=2, help="Value (name val…)")
-@click.option("-e", "--eval", "eval_", is_flag=True, help="Value must be evaluated")
-@click.option("-p", "--path", "path_", is_flag=True, help="Value is a path")
+@click.option("-v", "--var", nargs=2, multiple=True, help="Value (name val…)")
+@click.option("-e", "--eval", "eval_", nargs=2, multiple=True, help="Value (name val), evaluated")
+@click.option("-p", "--path", "path_", nargs=2, multiple=True, help="Value (name val), as path")
 @click.argument("path", nargs=1)
 @click.pass_obj
 async def set_(obj, path, code, tm, info, ok, repeat, delay, backoff, eval_, path_, var, copy):
-    """Save / modify a run entry."""
+    """Add or modify a runner.
+
+    Code typically requires some input parameters.
+
+    You should use '-v NAME VALUE' for string values, '-p NAME VALUE' for
+    paths, and '-e NAME VALUE' for other data. '-e NAME -' deletes an item.
+    """
     if obj.subpath[-1] == "-":
         raise click.UsageError("Group '-' can only be used for listing.")
-    if not path:
-        raise click.UsageError("You need a non-empty path.")
-    if path_ and eval_:
-        raise click.UsageError("'--eval' and '--path' are mutually exclusive.")
-    if (path_ or eval_) and not var:
-        raise click.UsageError("'--eval' or '--path' need a variable+value.")
 
     if code is not None:
         code = P(code)
@@ -304,14 +360,18 @@ async def set_(obj, path, code, tm, info, ok, repeat, delay, backoff, eval_, pat
         if copy and "code" not in res:
             raise click.UsageError("'--copy' needs a runner entry")
 
-    if var:
-        vl = res.setdefault("data", {})
-        k, v = var
-        if eval_:
-            v = eval(v)  # pylint:disable=eval-used
-        elif path_:
-            v = P(v)
-        vl[k] = v
+    vl = attrdict(**res.setdefault("data", {}))
+    for k, v in var:
+        vl = vl._update(P(k), v)
+    for k, v in eval_:
+        if v == "-":
+            vl = vl._delete(P(k))
+        else:
+            vl = vl._update(P(k), eval(v))  # pylint:disable=eval-used
+    for k, v in path_:
+        vl = vl._update(P(k), P(v))
+    res["data"] = vl
+
     if code is not None:
         res["code"] = code
     if ok is not None:
