@@ -260,8 +260,8 @@ class ErrorEntry(AttrClientEntry):
         if kid is not None:
             dkid = dest.get(self.root.name)
             # logger.warning("DEL 2 %r %r %r %r",self,dest,kid,dkid)
+            val = kid.get_value()
             if dkid is None:
-                val = kid.get_value()
                 dkid = dest.allocate(self.root.name)
                 await dkid.set_value(val)
                 await dkid.save()
@@ -487,7 +487,9 @@ class ErrorRoot(ClientRoot):
         try:
             await rec.save()
         except anyio.exceptions.ClosedResourceError:
-            logger.error("Could not save error %s %s: %s %r", subsystem, path, message, exc)
+            logger.exception(
+                "Could not save error %s %s: %s %r", subsystem, path, message, exc, exc_info=exc
+            )
             return  # owch, but can't be helped
 
         r = await rec.real_entry.add_exc(
