@@ -7,6 +7,7 @@ import anyio
 import weakref
 import heapq
 from collections.abc import Mapping
+from asyncscope import scope
 
 try:
     from contextlib import asynccontextmanager
@@ -213,7 +214,9 @@ class ClientEntry:
         This is a coroutine.
         """
         async with NoLock if _locked else self._lock:
-            r = await self.root.client.set(self._path, chain=self.chain, value=value, nchain=3)
+            r = await self.root.client.set(
+                self._path, chain=self.chain, value=value, nchain=3, idem=True
+            )
             if wait:
                 await self.root.wait_chain(r.chain)
             self.value = value
@@ -355,13 +358,14 @@ class ClientRoot(ClientEntry):
 
     CFG = "You need to override this with a dict(prefix=('where','ever'))"
 
-    def __init__(self, client, path, *, need_wait=False, cfg=None):
+    def __init__(self, client, path, *, need_wait=False, cfg=None, require_client=True):
         # pylint: disable=super-init-not-called
         self._init()
         self.client = client
         self._path = path
         self._need_wait = need_wait
         self._loaded = anyio.create_event()
+        self._require_client = require_client
 
         if cfg is None:
             cfg = {}
@@ -447,6 +451,9 @@ class ClientRoot(ClientEntry):
     async def run(self):
         """A coroutine that fetches, and continually updates, a subtree.
         """
+        if self._require_client:
+            scope.requires(self.client.scope)
+
         async with anyio.create_task_group() as tg:
             self._tg = tg
 
