@@ -8,7 +8,6 @@ import anyio
 from weakref import ref
 from asyncactor import NodeList
 from asyncactor import PingEvent, TagEvent, UntagEvent, AuthPingEvent
-from copy import deepcopy
 import psutil
 import time
 from collections.abc import Mapping
@@ -20,7 +19,7 @@ except ImportError:
 
 from .actor import ClientActor
 from .actor import DetachedState, PartialState, CompleteState, ActorState, BrokenState
-from .util import NotGiven, combine_dict, attrdict, P, Path, logger_for, spawn
+from .util import NotGiven, combine_dict, attrdict, P, Path, logger_for, spawn, digits
 
 from .exceptions import ServerError
 from .obj import AttrClientEntry, ClientRoot
@@ -67,6 +66,18 @@ class ChangeMsg(RunnerMsg):
     pass
 
 
+class MQTTmsg(RunnerMsg):
+    """A message transporting some MQTT data.
+
+    `value` is the MsgPack-decoded content. If that doesn't exist the
+    message is not decodeable.
+
+    The runner also sets the ``path`` attribute.
+    """
+
+    pass
+
+
 class ReadyMsg(RunnerMsg):
     """
     This message is queued when the last watcher has read all data.
@@ -97,6 +108,7 @@ for _c in (
     TimerMsg,
     ReadyMsg,
     ChangeMsg,
+    MQTTmsg,
     RunnerMsg,
     ErrorRecorded,
 ):
@@ -146,7 +158,9 @@ class CallAdmin:
                         await self.setup_done()
 
                     await tg.spawn(is_ok, oka)
+                await tg.spawn(self._changed_code, code)
 
+                await self._runner.send_event(ReadyMsg(0))
                 res = code(**data)
                 if code.is_async is not None:
                     res = await res
@@ -154,6 +168,13 @@ class CallAdmin:
                 await sc.cancel()
                 return res
 
+    async def _changed_code(self, code):
+        """
+        Kill the job if the underlying code has changed
+        """
+        await code.reload_event.wait()
+        await self.cancel()
+
     async def cancel(self):
         """
         Cancel the running task
@@ -256,6 +277,11 @@ class CallAdmin:
                 sc, self.scope = self.scope, None
                 await sc.cancel()
 
+        if isinstance(path, (tuple, list)):
+            path = Path.build(path)
+        elif not isinstance(path, Path):
+            raise RuntimeError(f"You didn't pass in a path: {path!r}")
+
         kw.setdefault("max_depth", 0)
         if kw.setdefault("fetch", True):
             self._n_watch += 1
@@ -264,19 +290,115 @@ class CallAdmin:
         await self._taskgroup.spawn(w.run)
         return w
 
+    async def send(self, path, value=NotGiven, raw=None):
+        """
+        Publish an MQTT message.
+
+        Set either ``value`` or ``raw``.
+        """
+        if isinstance(path, (tuple, list)):
+            path = Path.build(path)
+        elif not isinstance(path, Path):
+            raise RuntimeError(f"You didn't pass in a path: {path!r}")
+
+        await self._client.msg_send(topic=path, data=value, raw=raw)
+
+    async def set(self, path, value, chain=NotGiven):
+        """
+        Set a DistKV value.
+        """
+        if isinstance(path, (tuple, list)):
+            path = Path.build(path)
+        elif not isinstance(path, Path):
+            raise RuntimeError(f"You didn't pass in a path: {path!r}")
+
+        return await self._client.set(path, value=value, chain=chain)
+
+    async def get(self, path, value):
+        """
+        Get a DistKV value.
+        """
+        if isinstance(path, (tuple, list)):
+            path = Path.build(path)
+        elif not isinstance(path, Path):
+            raise RuntimeError(f"You didn't pass in a path: {path!r}")
+
+        res = await self._client.get(path, value=value)  # TODO chain
+
+        if "value" in res:
+            return res.value
+        else:
+            return KeyError(path)
+
+    async def monitor(self, path, cls=MQTTmsg, **kw):
+        """
+        Create an MQTT monitor.
+        Messages are encapsulated in `MQTTmsg` objects.
+
+        By default a monitor will only monitor a single entry. You may use
+        MQTT wildcards.
+
+        The message is decoded and stored in the ``value`` attribute unless
+        it's either undecodeable or ``raw`` is set, in which case it's
+        stored in ``.msg``. The topic the message was sent to is in
+        ``topic``.
+        """
+
+        class Monitor:
+            """Helper class for reading an MQTT topic"""
+
+            def __init__(self, admin, runner, client, cls, path, kw):
+                self.admin = admin
+                self.runner = runner
+                self.client = client
+                self.path = path
+                self.kw = kw
+                self.cls = cls
+                self.scope = None
+
+            async def run(self):
+                async with anyio.open_cancel_scope() as sc:
+                    self.scope = sc
+                    async with self.client.msg_monitor(path, **kw) as watcher:
+                        async for msg in watcher:
+                            if "topic" in msg:
+                                # pylint:disable=attribute-defined-outside-init
+                                chg = cls(msg.get("raw", None))
+                                try:
+                                    chg.value = msg.data
+                                except AttributeError:
+                                    pass
+                                chg.path = Path.build(msg.topic)
+                                await self.runner.send_event(chg)
+
+            async def cancel(self):
+                if self.scope is None:
+                    return False
+                sc, self.scope = self.scope, None
+                await sc.cancel()
+
+        if isinstance(path, (tuple, list)):
+            path = Path.build(path)
+        elif not isinstance(path, Path):
+            raise RuntimeError(f"You didn't pass in a path: {path!r}")
+
+        w = Monitor(self, self._runner, self._client, cls, path, kw)
+        await self._taskgroup.spawn(w.run)
+        return w
+
     async def timer(self, delay, cls=TimerMsg):
         class Timer:
-            def __init__(self, runner, delay, cls, tg):
+            def __init__(self, runner, cls, tg):
                 self.runner = runner
-                self.delay = delay
                 self.cls = cls
                 self.scope = None
                 self._taskgroup = tg
+                self.delay = None
 
             async def _run(self):
                 async with anyio.open_cancel_scope() as sc:
                     self.scope = sc
-                    await anyio.sleep(delay)
+                    await anyio.sleep(self.delay)
                     self.scope = None
                     await self.runner.send_event(self.cls(self))
 
@@ -290,10 +412,11 @@ class CallAdmin:
             async def run(self, delay):
                 await self.cancel()
                 self.delay = delay
-                await self._taskgroup.spawn(t._run)
+                if self.delay > 0:
+                    await self._taskgroup.spawn(t._run)
 
-        t = Timer(self._runner, delay, cls, self._taskgroup)
-        await self._taskgroup.spawn(t._run)
+        t = Timer(self._runner, cls, self._taskgroup)
+        await t.run(delay)
         return t
 
 
@@ -317,7 +440,7 @@ class RunnerEntry(AttrClientEntry):
         ok_after (float): the job is marked OK if it has run this long.
             Default: zero: the code will do that itself.
         backoff (float): Exponential back-off factor on errors.
-            Default: 1.4.
+            Default: 1.1.
 
     The code runs with these additional keywords::
 
@@ -338,7 +461,7 @@ class RunnerEntry(AttrClientEntry):
     delay = 100  # timedelta, before restarting
     repeat = 0
     target = 0
-    backoff = 1.4
+    backoff = 1.1
     ok_after = 0
 
     code = ()  # what to execute
@@ -376,11 +499,7 @@ class RunnerEntry(AttrClientEntry):
                 if state.node is not None:
                     raise RuntimeError(f"already running on {state.node}")
                 code = self.root.code.follow(self.code, create=False)
-                data = self.data
-                if data is None:
-                    data = {}
-                else:
-                    data = deepcopy(data)
+                data = combine_dict(self.data or {}, code.value.get("default", {}), deep=True)
 
                 if code.is_async:
                     data["_info"] = self._q = anyio.create_queue(QLEN)
@@ -390,6 +509,7 @@ class RunnerEntry(AttrClientEntry):
                 data["_P"] = P
                 data["_Path"] = Path
                 data["_log"] = self._logger
+                data["_digits"] = digits
 
                 state.started = time.time()
                 state.node = state.root.name
@@ -414,7 +534,7 @@ class RunnerEntry(AttrClientEntry):
         except ErrorRecorded:
             # record_error() has already been called
             self._comment = None
-            state.backoff += 1
+            state.backoff = min(state.backoff + 1, 20)
 
         except BaseException as exc:
             c, self._comment = self._comment, None
@@ -424,7 +544,7 @@ class RunnerEntry(AttrClientEntry):
                 )
                 if r is not None:
                     await self.root.err.wait_chain(r.chain)
-            state.backoff += 1
+            state.backoff = min(state.backoff + 1, 20)
         else:
             state.result = res
             state.backoff = 0
@@ -452,7 +572,8 @@ class RunnerEntry(AttrClientEntry):
     async def send_event(self, evt):
         """Send an event to the running process."""
         if self._q is None:
-            self._logger.info("Discarding %r", evt)
+            if self._running:
+                self._logger.info("Discarding %r", evt)
         elif self._q.qsize() < QLEN - 1:
             self._logger.debug("Event: %r", evt)
             await self._q.put(evt)
@@ -501,31 +622,31 @@ class RunnerEntry(AttrClientEntry):
 
         Returns:
           ``False``: No, it's running (or has run and doesn't restart).
-          n>0: wait for n seconds before thinking again.
-          n<0: should have been started n seconds ago, do something!
+          ``0``: No, it should not start
+          ``>0``: timestamp at which it should start, or should have started
 
         """
 
         state = self.state
         if self.code is None:
-            return False
+            return False, "no code"
         if state.node is not None:
-            return False
+            return False, "node set"
         if state.started and not state.stopped:
             raise RuntimeError("Running! should not be called")
 
         if self.target is None:
-            return False
+            return False, "no target"
         elif self.target > state.started:
-            return self.target
+            return self.target, "target > started"
         elif state.backoff:
-            return state.stopped + self.delay * (self.backoff ** state.backoff)
+            return state.stopped + self.delay * (self.backoff ** state.backoff), "backoff"
         elif self.repeat:
-            return state.stopped + self.repeat
+            return state.stopped + self.repeat, "repeat"
         elif state.started:
-            return False
+            return False, "is started"
         else:
-            return 0
+            return 0, "no target"
 
     def __hash__(self):
         return hash(self.subpath)
@@ -578,15 +699,19 @@ class StateEntry(AttrClientEntry):
       result (Any): the code's return value
       node (str): the node running this code
       backoff (float): on error, the multiplier to apply to the restart timeout
+      computed (float): computed start time
+      reason (str): reason why (not) starting
     """
 
-    ATTRS = "started stopped result node backoff".split()
+    ATTRS = "started stopped computed reason result node backoff".split()
 
     started = 0  # timestamp
     stopped = 0  # timestamp
     node = None  # on which the code is currently running
     result = NotGiven
     backoff = 0
+    computed = 0  # timestamp
+    reason = ""
 
     @property
     def runner(self):
@@ -605,7 +730,7 @@ class StateEntry(AttrClientEntry):
 
         self.stopped = time.time()
         self.node = None
-        self.backoff += 1
+        self.backoff = min(20, self.backoff + 1)
         await self.root.runner.err.record_error(
             "run", self.runner._path, message="Runner restarted"
         )
@@ -613,14 +738,14 @@ class StateEntry(AttrClientEntry):
 
     async def stale(self):
         self.stopped = time.time()
+        node = self.node
         self.node = None
-        self.backoff += 2
+        self.backoff = min(20, self.backoff + 2)
         await self.root.runner.err.record_error(
             "run",
             self.runner._path,
-            client_name=self.node,
-            message="Runner {node} {state}",
-            data={"node": self.node, "state": "offline" if self.stopped else "stale"},
+            message="Runner killed: {node} {state}",
+            data={"node": node, "state": "offline" if self.stopped else "stale"},
         )
         await self.save()
 
@@ -635,6 +760,9 @@ class StateEntry(AttrClientEntry):
         except KeyError:
             return
 
+        # side effect: add to the global node list
+        run.root.get_node(n)
+
         # Check whether running code needs to be killed off
         if run.scope is None:
             return
@@ -720,6 +848,7 @@ class _BaseRunnerRoot(ClientRoot):
 
     _trigger: anyio.abc.Event = None
     _run_now_task: anyio.abc.CancelScope = None
+    _tagged: bool = True
 
     err = None
     code = None
@@ -792,8 +921,6 @@ class _BaseRunnerRoot(ClientRoot):
         await self._tg.spawn(self._run_actor)
 
         # the next block needs to be atomic
-        for n in self.all_children:
-            print(n)
         self.ready = True
 
         await self.state.runner_running()
@@ -827,16 +954,19 @@ class _BaseRunnerRoot(ClientRoot):
                         self._trigger = anyio.create_event()
 
                 t = time.time()
-                t_next = t + 99999
+                t_next = t + 999
                 for j in self.all_children:
-                    d = j.should_start()
-                    if d is False:
+                    d, r = j.should_start()
+                    if not d or d > t:
+                        j.state.computed = d
+                        j.state.reason = r
+                        if self._tagged:
+                            await j.state.save()
+                        if d and t_next > d:
+                            t_next = d
                         continue
-                    if d <= t:
-                        await self._tg.spawn(j.run)
-                        await anyio.sleep(self._start_delay)
-                    elif t_next > d:
-                        t_next = d
+                    await self._tg.spawn(j.run)
+                    await anyio.sleep(self._start_delay)
 
     async def notify_actor_state(self, msg=None):
         """
@@ -874,10 +1004,13 @@ class AnyRunnerRoot(_BaseRunnerRoot):
     _stale_times = None
     tg = None
     seen_load = None
+    _tagged: bool = False
 
     def __init__(self, *a, **kw):
         super().__init__(*a, **kw)
-        self.group = "run." + self._name
+        self.group = (
+            P(self.client.config.server.root) + P(self._cfg["name"]) | "any" | self._path[-1]
+        )
 
     def get_node(self, name):
         return RunnerNode(self, name)
@@ -885,7 +1018,9 @@ class AnyRunnerRoot(_BaseRunnerRoot):
     @property
     def max_age(self):
         """Timeout after which we really should have gotten another go"""
-        return self._act.cycle_time_max * (self._act.history_size + 1.5)
+        # allow one go-around without being in the list; happens when the
+        # bus is slow
+        return self._act.cycle_time_max * (self._act.history_size + 2.5)
 
     async def _run_actor(self):
         """
@@ -916,11 +1051,13 @@ class AnyRunnerRoot(_BaseRunnerRoot):
                     self.node_history += node
 
                 elif isinstance(msg, TagEvent):
+                    self._tagged = True
                     load = 100 - psutil.cpu_percent(interval=None)
                     await act.set_value(load)
                     if self.seen_load is not None:
                         pass  # TODO
 
+                    self.get_node(self.name).seen = time.time()
                     self.node_history += self.name
                     evt = anyio.create_event()
                     await self.spawn(self._run_now, evt)
@@ -930,6 +1067,7 @@ class AnyRunnerRoot(_BaseRunnerRoot):
                     await self.state.ping()
 
                 elif isinstance(msg, UntagEvent):
+                    self._tagged = False
                     await act.set_value(100 - psutil.cpu_percent(interval=None))
                     self.seen_load = 0
 
@@ -970,28 +1108,19 @@ class AnyRunnerRoot(_BaseRunnerRoot):
 
         TODO check where to use time.monotonic
         """
+        t0 = None
         t1 = time.time()
-        while age_q is not None:
-            await self.find_stale_nodes(t1)
+        while True:
             async with anyio.move_on_after(self.max_age):
                 await age_q.get()
                 t1 = time.time()
-                continue
             t2 = time.time()
             if t1 + self.max_age < t2:
                 raise NotSelected(self.max_age, t2, t1)
+            t0 = t1
             t1 = t2
-
-    async def _cleanup_nodes(self):
-        t = time.time()
-        while len(self.node_history) > 1:
-            node = self.get_node(self.node_history[-1])
-            if t - node.seen < self.max_age:
-                break
-            assert node.name == self.node_history.pop()
-            for j in self.all_children:
-                if j.node == node.name:
-                    await j.seems_down()
+            if t0 is not None:
+                await self.find_stale_nodes(t0)
 
 
 class SingleRunnerRoot(_BaseRunnerRoot):
@@ -1031,7 +1160,12 @@ class SingleRunnerRoot(_BaseRunnerRoot):
 
     def __init__(self, *a, **kw):
         super().__init__(*a, **kw)
-        self.group = "run." + self._path[-2]
+        self.group = (
+            P(self.client.config.server.root) + P(self._cfg["name"])
+            | "single"
+            | self._path[-2]
+            | self._path[-1]
+        )
 
     async def set_value(self, value):
         await super().set_value(value)
@@ -1104,7 +1238,9 @@ class AllRunnerRoot(SingleRunnerRoot):
 
     def __init__(self, *a, **kw):
         super().__init__(*a, **kw)
-        self.group = "run." + self.name
+        self.group = (
+            P(self.client.config.server.root) + P(self._cfg["name"]) | "all" | self._path[-1]
+        )
 
     async def _state_runner(self):
         self.state = await StateRoot.as_handler(
