@@ -579,9 +579,9 @@ class SCmd_msg_monitor(StreamCommand):
         raw = msg.get("raw", False)
         topic = msg.topic
         if isinstance(topic, str):
-            topic = (topic,)
+            topic = P(topic)
         if len(topic) and topic[0][0] == ":":
-            topic = self.client.server.cfg.root + topic
+            topic = P(self.client.server.cfg.root) + topic
 
         async with self.client.server.serf.monitor(*topic) as stream:
             async for resp in stream:
@@ -920,18 +920,28 @@ class ServerClient:
                 # TODO pass exceptions to the client
 
         send_prev = True
+        nchain = msg.get("nchain", 1)
+
+        if msg.get("idem", False) and type(entry.data) is type(value) and entry.data == value:
+            res = attrdict(tock=entry.tock, changed=False)
+            if nchain > 0:
+                res.chain = entry.chain.serialize(nchain=nchain)
+            return res
+
         if "prev" in msg:
             if entry.data != msg.prev:
-                raise ClientError(f"Data is {entry.data !r}")
+                raise ClientError(f"Data is {entry.data !r} not {msg.prev !r} at {msg.path}")
             send_prev = False
         if "chain" in msg:
             if msg.chain is None:
                 if entry.data is not NotGiven:
-                    raise ClientChainError("This entry already exists")
+                    raise ClientChainError("Entry already exists at {msg.path}")
             elif entry.data is NotGiven:
-                raise ClientChainError("This entry is new")
+                raise ClientChainError("Entry is new at {msg.path}")
             elif entry.chain != msg.chain:
-                raise ClientChainError(f"Chain is {entry.chain !r}")
+                raise ClientChainError(
+                    f"Chain is {entry.chain !r} not {msg.chain !r} for {msg.path}"
+                )
             send_prev = False
 
         res = attrdict()
@@ -995,7 +1005,7 @@ class ServerClient:
         if isinstance(topic, str):
             topic = (topic,)
         if topic[0][0] == ":":
-            topic = self.server.cfg.root + topic
+            topic = P(self.server.cfg.root) + topic
         if "raw" in msg:
             assert "data" not in msg
             data = msg.raw
@@ -2004,6 +2014,7 @@ class Server:
 
                 cfg["auth"] = gen_auth(auth)
 
+                self.logger.error("Sync: connecting: %s", cfg)
                 async with distkv_client.open_client(connect=cfg) as client:
                     # TODO auth this client
 
@@ -2062,6 +2073,7 @@ class Server:
                         self.fetch_missing.add(nst)
                 if len(self.fetch_missing):
                     self.fetch_running = False
+                    self.logger.error("Sync: missing: %s", self.fetch_missing)
                     await self.spawn(self.do_send_missing)
                 if self.force_startup or not len(self.fetch_missing):
                     if self.node.tick is None:
