@@ -17,10 +17,13 @@ import ast as _ast
 from getpass import getpass
 from collections import deque
 from collections.abc import Mapping
+from copy import deepcopy
 from types import ModuleType
 from typing import Union, Dict, Optional
 from ssl import SSLContext
 from functools import partial, total_ordering
+from math import log10
+from sniffio import current_async_library
 
 import ruamel.yaml as yaml
 
@@ -122,7 +125,7 @@ class NotGiven:
         return "NotGiven"
 
 
-def combine_dict(*d, cls=dict) -> dict:
+def combine_dict(*d, cls=dict, deep=False) -> dict:
     """
     Returns a dict with all keys+values of all dict arguments.
     The first found value wins.
@@ -132,27 +135,40 @@ def combine_dict(*d, cls=dict) -> dict:
     Args:
       cls (type): a class to instantiate the result with. Default: dict.
         Often used: :class:`attrdict`.
+      deep (bool): if set, always copy.
     """
     res = cls()
     keys = {}
-    if len(d) <= 1:
-        return d
+    if not len(d):
+        return res
+
+    if len(d) == 1 and deep and not isinstance(d[0], Mapping):
+        if deep and isinstance(d[0], (list, tuple)):
+            return deepcopy(d[0])
+        else:
+            return d[0]
+
     for kv in d:
         for k, v in kv.items():
             if k not in keys:
                 keys[k] = []
             keys[k].append(v)
+
     for k, v in keys.items():
         if v[0] is NotGiven:
             res.pop(k, None)
-        elif len(v) == 1:
+        elif len(v) == 1 and not deep:
             res[k] = v[0]
         elif not isinstance(v[0], Mapping):
             for vv in v[1:]:
                 assert vv is NotGiven or not isinstance(vv, Mapping)
-            res[k] = v[0]
+            if deep and isinstance(v[0], (list, tuple)):
+                res[k] = deepcopy(v[0])
+            else:
+                res[k] = v[0]
         else:
             res[k] = combine_dict(*v, cls=cls)
+
     return res
 
 
@@ -477,16 +493,20 @@ class _Server:
                 await server.aclose()
 
     async def __aenter__(self):
+        if current_async_library() != "trio":
+            raise RuntimeError("This only works with Trio right now.Sorry.")
         send_q, self.recv_q = trio.open_memory_channel(1)
         try:
             servers = await trio.open_tcp_listeners(self.port, **self._kw)
         except EnvironmentError as exc:
-            raise OSError(f"Port {self.port} in use", errno=exc.errno) from exc
+            err = OSError(f"Port {self.port} in use")
+            err.errno = exc.errno
+            raise err from exc
 
         self.ports = []
-        for s in servers:
-            await self.tg.spawn(self._accept, s, send_q.clone())
-        await send_q.aclose()
+        async with send_q:
+            for s in servers:
+                await self.tg.spawn(self._accept, s, send_q.clone())
         return self
 
     async def __aexit__(self, *tb):
@@ -1059,6 +1079,8 @@ class Path(collections.abc.Sequence):
         # 2=it's a hex number
 
         pos = 0
+        if isinstance(path, (tuple, list)):
+            return cls.build(path)
         if path == ":":
             return cls()
 
@@ -1325,7 +1347,7 @@ def res_delete(res, attr: Path, **kw):  # pylint: disable=redefined-outer-name
 
 def logger_for(path: Path):
     """
-    Create a logger for this path.
+    Create a logger for this ``path``.
     """
     if not len(path):
         p = "distkv.root"
@@ -1338,3 +1360,20 @@ def logger_for(path: Path):
     if len(path) > 1:
         p += "." + ".".join(path[1:])
     return logging.getLogger(p)
+
+
+def digits(n, digits=6):  # pylint: disable=redefined-outer-name
+    """
+    Returns ``n`` rounded to ``digits`` significant digits. Default: 6.
+    Ensures that the number doesn't carry nonsense precision or
+    floating-point artefacts.
+
+    >>> digits(123456789, 4)
+    123400000
+    >>> digits(math.pi, 4)
+    3.142
+
+    ``digits`` may be a fraction, in order to move the cut-off point to
+    somewhere other than between 9.999 and 10.00.
+    """
+    return round(n, int(digits - 1 - log10(abs(n))))
