@@ -2,10 +2,11 @@ import pytest
 import io
 from functools import partial
 
-from .mock_mqtt import stdtest
-from .run import run
+from distkv.mock import run
+from distkv.mock.mqtt import stdtest
+
 from distkv.client import ServerError
-from distkv.util import PathLongener
+from distkv.util import PathLongener, P
 
 import logging
 
@@ -30,7 +31,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int"),
+                    path=P("type.int"),
                     value={
                         "bad": ["foo", None],
                         "good": [0, 1, 2],
@@ -41,7 +42,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int"),
+                    path=P("type.int"),
                     value={
                         "bad": ["foo", None],
                         "good": [0, 1, "dud"],
@@ -51,7 +52,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int"),
+                    path=P("type.int"),
                     value={
                         "bad": ["foo", 4],
                         "good": [0, 1, 2],
@@ -61,7 +62,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int"),
+                    path=P("type.int"),
                     value={
                         "bad": [],
                         "good": [0, 1, 2],
@@ -71,7 +72,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int"),
+                    path=P("type.int"),
                     value={
                         "bad": ["foo", None],
                         "good": [1],
@@ -80,7 +81,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
                 )
             await c._request(
                 "set_internal",
-                path=("type", "int"),
+                path=P("type.int"),
                 value={
                     "bad": ["foo", None],
                     "good": [0, 1, 2],
@@ -90,7 +91,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int", "percent"),
+                    path=P("type.int.percent"),
                     value={
                         "bad": ["fuf", 101],
                         "good": [0, 55, 100],
@@ -100,7 +101,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             with pytest.raises(ServerError):
                 await c._request(
                     "set_internal",
-                    path=("type", "int", "percent"),
+                    path=P("type.int.percent"),
                     value={
                         "bad": ["fuf", 101],
                         "good": [0, 5.5, 100],
@@ -109,7 +110,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
                 )
             await c._request(
                 "set_internal",
-                path=("type", "int", "percent"),
+                path=P("type.int.percent"),
                 value={
                     "bad": [-1, 101],
                     "good": [0, 55, 100],
@@ -118,22 +119,18 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
             )
             with pytest.raises(ServerError):
                 await c._request(
-                    "set_internal",
-                    path=("match", "one", "+", "two"),
-                    value={"tope": ("int", "percent")},
+                    "set_internal", path=P("match.one.+.two"), value={"tope": P("int.percent")}
                 )
             await c._request(
-                "set_internal",
-                path=("match", "one", "+", "two"),
-                value={"type": ("int", "percent")},
+                "set_internal", path=P("match.one.+.two"), value={"type": P("int.percent")}
             )
 
-            await c.set("one", "x", "two", value=99)
+            await c.set(P("one.x.two"), value=99)
             with pytest.raises(ServerError):
-                await c.set("one", "y", "two", value=9.9)
+                await c.set(P("one.y.two"), value=9.9)
             with pytest.raises(ServerError):
-                await c.set("one", "y", "two", value="zoz")
-            await c.set("one", "y", value="zoz")
+                await c.set(P("one.y.two"), value="zoz")
+            await c.set(P("one.y"), value="zoz")
 
             pass  # client end
         pass  # server end
@@ -142,7 +139,7 @@ async def test_71_basic(autojump_clock):  # pylint: disable=unused-argument
 @pytest.mark.trio
 async def test_72_cmd(autojump_clock, tmpdir):  # pylint: disable=unused-argument
     async with stdtest(args={"init": 123}, tocks=80) as st:
-        s, = st.s
+        (s,) = st.s
         async with st.client() as c:
             h = p = None  # pylint
             for h, p, *_ in s.ports:
@@ -183,8 +180,7 @@ code: "if not isinstance(value,int): raise ValueError('not an int')"
                     "-1",
                     "-b",
                     "5.5",
-                    "int",
-                    "percent",
+                    "int.percent",
                 )
             await rr(
                 "type",
@@ -201,21 +197,20 @@ code: "if not isinstance(value,int): raise ValueError('not an int')"
                 "-1",
                 "-b",
                 "555",
-                "int",
-                "percent",
+                "int.percent",
             )
 
-            await rr("type", "match", "-t", "int", "-t", "percent", "foo", "+", "bar")
+            await rr("type", "match", "-t", "int.percent", "foo.+.bar")
 
             with pytest.raises(ServerError):
-                await rr("data", "set", "-v", "123", "foo", "dud", "bar")
+                await rr("data", "set", "-v", "123", "foo.dud.bar")
             with pytest.raises(ServerError):
-                await rr("data", "set", "-ev", "123", "foo", "dud", "bar")
+                await rr("data", "set", "-ev", "123", "foo.dud.bar")
             with pytest.raises(ServerError):
-                await rr("data", "set", "-ev", "5.5", "foo", "dud", "bar")
-            await rr("data", "set", "-ev", "55", "foo", "dud", "bar")
+                await rr("data", "set", "-ev", "5.5", "foo.dud.bar")
+            await rr("data", "set", "-ev", "55", "foo.dud.bar")
 
-            assert (await c.get("foo", "dud", "bar")).value == 55
+            assert (await c.get(P("foo.dud.bar"))).value == 55
 
             pass  # client end
         pass  # server end
